package com.bofeng.security;

import com.bofeng.Consts;
import com.bofeng.JwtHelper;
import com.bofeng.service.SecurityService;
import com.google.common.base.Strings;
import com.yvan.ModelOps;
import com.yvan.mvc.ErrorController;
import com.yvan.platform.YvanUtil;
import com.yvan.springmvc.HttpUtils;
import com.yvan.springmvc.ResponseUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authz.UnauthenticatedException;
import org.apache.shiro.util.AntPathMatcher;
import org.apache.shiro.web.filter.authc.BasicHttpAuthenticationFilter;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Slf4j
public class JwtFilter extends BasicHttpAuthenticationFilter {

    private static final AntPathMatcher ANT_PATH_MATCHER = new AntPathMatcher();
    private final SecurityService securityService;

    public JwtFilter(SecurityService securityService) {
        this.securityService = securityService;
    }

    /**
     * 是不是已经登录
     */
    @Override
    protected boolean isLoginAttempt(ServletRequest request, ServletResponse response) {
        return !Strings.isNullOrEmpty(JwtHelper.getAuthHeader(request));
    }

    /**
     * 对票据进行延期
     */
    @Override
    protected boolean executeLogin(ServletRequest request, ServletResponse response) throws Exception {
        String authorization = JwtHelper.getAuthHeader(request);
        JwtToken token = new JwtToken(authorization);
        getSubject(request, response).login(token);
        return true;
    }

    /**
     * 这里我们详细说明下为什么最终返回的都是true，即允许访问
     * 例如我们提供一个地址 GET /article
     * 登入用户和游客看到的内容是不同的
     * 如果在这里返回了false，请求会被直接拦截，用户看不到任何东西
     * 所以我们在这里返回true，Controller中可以通过 subject.isAuthenticated() 来判断用户是否登入
     * 如果有些资源只有登入用户才能访问，我们只需要在方法上面加上 @RequiresAuthentication 注解即可
     * 但是这样做有一个缺点，就是不能够对GET,POST等请求进行分别过滤鉴权(因为我们重写了官方的方法)，但实际上对应用影响不大
     */
    @Override
    protected boolean isAccessAllowed(ServletRequest request, ServletResponse response, Object mappedValue) {
        final HttpServletRequest r = (HttpServletRequest) request;

        for (String path : Consts.NOT_SECURITY_PATHS) {
            if (ANT_PATH_MATCHER.match(r.getContextPath() + path, ((HttpServletRequest) request).getRequestURI())) {
                return true;
            }
        }

        if (isLoginAttempt(request, response)) {
            try {
                executeLogin(request, response);
            } catch (Exception e) {
                //response401(request, response);
                return false;
            }
        }
        //return true;
        return true;
    }

    @Override
    protected boolean onLoginFailure(AuthenticationToken token, AuthenticationException e, ServletRequest request, ServletResponse response) {
        try {
            ((HttpServletResponse) response).setStatus(HttpStatus.BAD_REQUEST.value());
            response.getWriter().print("账号活密码错误");
        } catch (IOException e1) {
            log.error(e1.getMessage(), e1);
        }
        return false;
    }

    /**
     * 当访问拒绝时是否已经处理了；
     * 如果返回true表示需要继续处理；
     * 如果返回false表示该拦截器实例已经处理完成了，将直接返回即可。
     **/
    @Override
    protected boolean onAccessDenied(ServletRequest request, ServletResponse response) throws Exception {
        HttpServletRequest httpServletRequest = (HttpServletRequest) request;
        HttpServletResponse httpServletResponse = (HttpServletResponse) response;

        HttpUtils.removeCookie(
                httpServletRequest,
                httpServletResponse,
                JwtHelper.AUTH_COOKIE_NAME,
                "/");
        httpServletResponse.setHeader("Access-Control-Expose-Headers", JwtHelper.AUTH_HEADER_NAME);
        httpServletResponse.setHeader(JwtHelper.AUTH_HEADER_NAME, "remove");

        response401(request, response);
        return false;
    }


    /**
     * 对跨域提供支持
     */
    @Override
    protected boolean preHandle(ServletRequest request, ServletResponse response) throws Exception {
        HttpServletRequest httpServletRequest = (HttpServletRequest) request;
        HttpServletResponse httpServletResponse = (HttpServletResponse) response;
        httpServletResponse.setHeader("Access-control-Allow-Origin", httpServletRequest.getHeader("Origin"));
        httpServletResponse.setHeader("Access-Control-Allow-Methods", "GET,POST");
        httpServletResponse.setHeader("Access-Control-Allow-Headers", httpServletRequest.getHeader("Access-Control-Request-Headers"));
        // 跨域时会首先发送一个option请求，这里我们给option请求直接返回正常状态
        if (httpServletRequest.getMethod().equals(RequestMethod.OPTIONS.name())) {
            httpServletResponse.setStatus(HttpStatus.OK.value());
            return false;
        }
        return super.preHandle(request, response);
    }

    /**
     * 将非法请求跳转到 /401
     */
    private void response401(ServletRequest req, ServletResponse resp) {
        HttpServletRequest request = (HttpServletRequest) req;
        ModelOps errorData = ErrorController.PROCESS_ERROR(request, new UnauthenticatedException("401"));
        HttpServletResponse r = ((HttpServletResponse) resp);

        ResponseUtils.writeJson(r, YvanUtil.toJson(errorData));
    }
}
